<?php

namespace Drupal\fillpdf_comprehensive_mapper;

use Drupal\Core\Batch\BatchBuilder;

/**
 * Methods for running the bulk update of all other FillPDF maps in a batch.
 */
class BulkUpdate {

  public static function getFillPdfFormIds(int $source_fid) {
    return \Drupal::entityQuery('fillpdf_form')->condition('fid', $source_fid, '<>')->accessCheck(TRUE)->execute();
  }

  public static function initBatch() {
    $source_fid = (int) \Drupal::config('fillpdf_comprehensive_mapper.settings')->get('source_form');
    $fids = self::getFillPdfFormIds($source_fid);
    // If we do not have anything to batch, we can end this early.
    if (!$fids) {
      return;
    }
    $batch = new BatchBuilder();
    // Using string instead of a use statement and BulkUpdateBatch::class below
    // because presumably that delays loading code until it is actually used?
    $batch->setTitle('Updating all FillPDF forms from comprehensive mapping form.')
      ->setFinishCallback(['Drupal\fillpdf_comprehensive_mapper\BulkUpdateBatch', 'finish'])
      ->setInitMessage('Beginning')
      ->setProgressMessage('Processing…')
      ->setErrorMessage('An error occurred during processing.');
    $chunks = array_chunk($fids, 20);
    foreach ($chunks as $id => $chunk) {
      $args = [$id, $chunk, $source_fid];
      $batch->addOperation(['Drupal\fillpdf_comprehensive_mapper\BulkUpdateBatch', 'process'], $args);
    }
    batch_set($batch->toArray());
  }

}