<?php

namespace Drupal\fillpdf_comprehensive_mapper\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\fillpdf_comprehensive_mapper\BulkUpdate;
use Drupal\fillpdf_comprehensive_mapper\Service\FormsUpdater;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure FillPDF settings form.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * Constructs a FillPdfSettingsForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Config\TypedConfigManagerInterface $typedConfigManager
   *   The typed config manager.
   * @param \Drupal\fillpdf\Service\FillPdfAdminFormHelper $admin_form_helper
   *   The FillPDF admin form helper service.
   */
  public function __construct(
    protected ConfigFactoryInterface $config_factory,
    protected TypedConfigManagerInterface $typedConfigManager,
    protected FormsUpdater $formsUpdater,
  ) {
    parent::__construct($config_factory, $typedConfigManager);

    $this->formsUpdater = $formsUpdater;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('config.typed'),
      $container->get('fillpdf_comprehensive_mapper.forms_updater'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'fillpdf_comprehensive_mapper_settings';
  }

  /**
   * Gets the configuration names that will be editable.
   *
   * @return array
   *   An array of configuration object names that are editable if called in
   *   conjunction with the trait's config() method.
   */
  protected function getEditableConfigNames() {
    return ['fillpdf_comprehensive_mapper.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);

    $config = $this->config('fillpdf_comprehensive_mapper.settings');

    $ids = \Drupal::entityQuery('fillpdf_form')->accessCheck(TRUE)->execute();
    // TODO do an entity load to get the objects so we can use their names in this dropdown.
    $fillpdf_form_options = $ids;
    $form['source_form'] = [
      '#type' => 'select',
      '#title' => $this->t("Source FillPDF form"),
      '#description' => $this->t("The FillPDF form to use to as the comprehensive import mapping for all other FillPDF forms."),
      '#options' => $fillpdf_form_options,
      '#default_value' => $config->get('source_form') ?: '',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Save form values.
    $values = $form_state->getValues();
    $config = $this->config('fillpdf_comprehensive_mapper.settings');

    $config->set('source_form', $values['source_form']);

    $config->save();

    // Initiate a batch update of all forms to use the new one.  (We are not
    // yet checking if it changed but that is the only thing on this form.)
    BulkUpdate::initBatch();

    parent::submitForm($form, $form_state);
  }

}
