<?php

namespace Drupal\fillpdf_comprehensive_mapper\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\fillpdf\Entity\FillPdfForm;
use Drupal\fillpdf\Serializer;
use Drupal\fillpdf\SerializerInterface;
use Drupal\fillpdf_comprehensive_mapper\FormsUpdaterInterface;
use Psr\Log\LoggerInterface;

class FormsUpdater implements FormsUpdaterInterface {

  /**
   * Constructs a new FormsUpdater service.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   */
  public function __construct(protected ConfigFactoryInterface $configFactory, protected SerializerInterface $fillpdfSerializer, protected LoggerInterface $logger) {
    $this->configFactory = $configFactory;
    $this->fillpdfSerializer = $fillpdfSerializer;
    $this->logger = $logger;
  }

  /**
   * Update all FillPDF forms from the designated comprehensive mapper form.
   */
  public function overwriteAllWithMapper(): void {
    $mapper_id = $this->configFactory->get('fillpdf_comprehensive_mapper.settings')->get('source_form');
    if (!$mapper_id) {
      $this->logger->warning(t('No source form for mapping set.'));
      return;
    }
    $mapper_form = FillPdfForm::load($mapper_id);
    if (!$mapper_form) {
      $this->logger->warning(t('No valid FillPDF form for source form ID %mapper_id.', ['%mapper_id' => $mapper_id]));
      return;
    }

    $forms = FillPdfForm::loadMultiple();
    // Let's not re-map ourselves.
    unset($forms[$mapper_id]);

    $keyed_fields = $mapper_form->getFormFields();
    foreach ($forms as $fillpdf_form) {
      $existing_fields = $fillpdf_form->getFormFields();
      $unmatched_pdf_keys = $this->fillpdfSerializer->importFormFields($keyed_fields, $existing_fields);
    }
  }
}